@@ -666,15 +666,27 @@ SAAS_PACKAGE = {
     ]
 }
 
+activePreset1 = {
+    'description': 'Single Xeon 1270, 8GB Ram, 2x1TB SATA disks, Non-RAID',
+    'id': 64,
+    'isActive': '1',
+    'keyName': 'S1270_8GB_2X1TBSATA_NORAID',
+    'name': 'S1270 8GB 2X1TBSATA NORAID',
+    'packageId': 200
+}
+
+activePreset2 = {
+    'description': 'Dual Xeon Gold, 384GB Ram, 4x960GB SSD, RAID 10',
+    'id': 65,
+    'isActive': '1',
+    'keyName': 'DGOLD_6140_384GB_4X960GB_SSD_SED_RAID_10',
+    'name': 'DGOLD 6140 384GB 4X960GB SSD SED RAID 10',
+    'packageId': 200
+}
+
 getAllObjects = [{
-    'activePresets': [{
-        'description': 'Single Xeon 1270, 8GB Ram, 2x1TB SATA disks, Non-RAID',
-        'id': 64,
-        'isActive': '1',
-        'keyName': 'S1270_8GB_2X1TBSATA_NORAID',
-        'name': 'S1270 8GB 2X1TBSATA NORAID',
-        'packageId': 200
-    }],
+    'activePresets': [activePreset1],
+    'accountRestrictedActivePresets': [activePreset2],
     'description': 'Bare Metal Server',
     'firstOrderStepId': 1,
     'id': 200,
@@ -1121,3 +1133,21 @@ getAllObjectsDH = [{
     "isActive": 1,
     "description": "Dedicated Host"
 }]
+
+getRegions = [{
+    "description": "WDC07 - Washington, DC",
+    "keyname": "WASHINGTON07",
+    "locations": [{
+        "location": {
+            "euCompliantFlag": False,
+            "id": 2017603,
+            "longName": "Washington 7",
+            "name": "wdc07",
+            "statusId": 2},
+        "locationPackageDetails": [{
+            "isAvailable": 1,
+            "locationId": 2017603,
+            "packageId": 46
+        }]
+    }]
+}]
