@@ -7,10 +7,11 @@
 """
 import logging
 import socket
+import time
+
 
 import SoftLayer
 from SoftLayer.decoration import retry
-from SoftLayer import exceptions
 from SoftLayer.managers import ordering
 from SoftLayer import utils
 
@@ -56,8 +57,7 @@ class HardwareManager(utils.IdentifierMixin, object):
         else:
             self.ordering_manager = ordering_manager
 
-    def cancel_hardware(self, hardware_id, reason='unneeded', comment='',
-                        immediate=False):
+    def cancel_hardware(self, hardware_id, reason='unneeded', comment='', immediate=False):
         """Cancels the specified dedicated server.
 
         Example::
@@ -66,29 +66,48 @@ class HardwareManager(utils.IdentifierMixin, object):
             result = mgr.cancel_hardware(hardware_id=1234)
 
         :param int hardware_id: The ID of the hardware to be cancelled.
-        :param string reason: The reason code for the cancellation. This should
-                              come from :func:`get_cancellation_reasons`.
-        :param string comment: An optional comment to include with the
-                               cancellation.
+        :param string reason: The reason code for the cancellation. This should come from
+                              :func:`get_cancellation_reasons`.
+        :param string comment: An optional comment to include with the cancellation.
+        :param bool immediate: If set to True, will automatically update the cancelation ticket to request
+                               the resource be reclaimed asap. This request still has to be reviewed by a human
+        :returns: True on success or an exception
         """
 
         # Get cancel reason
         reasons = self.get_cancellation_reasons()
         cancel_reason = reasons.get(reason, reasons['unneeded'])
+        ticket_mgr = SoftLayer.TicketManager(self.client)
+        mask = 'mask[id, hourlyBillingFlag, billingItem[id], openCancellationTicket[id]]'
+        hw_billing = self.get_hardware(hardware_id, mask=mask)
 
-        hw_billing = self.get_hardware(hardware_id,
-                                       mask='mask[id, billingItem.id]')
         if 'billingItem' not in hw_billing:
-            raise SoftLayer.SoftLayerError(
-                "No billing item found for hardware")
+            raise SoftLayer.SoftLayerError("Ticket #%s already exists for this server" %
+                                           hw_billing['openCancellationTicket']['id'])
 
         billing_id = hw_billing['billingItem']['id']
 
-        return self.client.call('Billing_Item', 'cancelItem',
-                                immediate, False, cancel_reason, comment,
-                                id=billing_id)
+        if immediate and not hw_billing['hourlyBillingFlag']:
+            LOGGER.warning("Immediate cancelation of montly servers is not guaranteed. " +
+                           "Please check the cancelation ticket for updates.")
 
-    @retry(exceptions.SoftLayerAPIError, logger=LOGGER)
+            result = self.client.call('Billing_Item', 'cancelItem',
+                                      False, False, cancel_reason, comment, id=billing_id)
+            hw_billing = self.get_hardware(hardware_id, mask=mask)
+            ticket_number = hw_billing['openCancellationTicket']['id']
+            cancel_message = "Please reclaim this server ASAP, it is no longer needed. Thankyou."
+            ticket_mgr.update_ticket(ticket_number, cancel_message)
+            LOGGER.info("Cancelation ticket #%s has been updated requesting immediate reclaim", ticket_number)
+        else:
+            result = self.client.call('Billing_Item', 'cancelItem',
+                                      immediate, False, cancel_reason, comment, id=billing_id)
+            hw_billing = self.get_hardware(hardware_id, mask=mask)
+            ticket_number = hw_billing['openCancellationTicket']['id']
+            LOGGER.info("Cancelation ticket #%s has been created", ticket_number)
+
+        return result
+
+    @retry(logger=LOGGER)
     def list_hardware(self, tags=None, cpus=None, memory=None, hostname=None,
                       domain=None, datacenter=None, nic_speed=None,
                       public_ip=None, private_ip=None, **kwargs):
@@ -176,7 +195,7 @@ class HardwareManager(utils.IdentifierMixin, object):
         kwargs['filter'] = _filter.to_dict()
         return self.account.getHardware(**kwargs)
 
-    @retry(exceptions.SoftLayerAPIError, logger=LOGGER)
+    @retry(logger=LOGGER)
     def get_hardware(self, hardware_id, **kwargs):
         """Get details about a hardware device.
 
@@ -298,7 +317,7 @@ class HardwareManager(utils.IdentifierMixin, object):
 
         See get_create_options() for valid arguments.
 
-        :param string size: server size name
+        :param string size: server size name or presetId
         :param string hostname: server hostname
         :param string domain: server domain name
         :param string location: location (datacenter) name
@@ -343,7 +362,7 @@ class HardwareManager(utils.IdentifierMixin, object):
             'moving': 'Moving to competitor',
         }
 
-    @retry(exceptions.SoftLayerAPIError, logger=LOGGER)
+    @retry(logger=LOGGER)
     def get_create_options(self):
         """Returns valid options for ordering hardware."""
 
@@ -359,7 +378,7 @@ class HardwareManager(utils.IdentifierMixin, object):
 
         # Sizes
         sizes = []
-        for preset in package['activePresets']:
+        for preset in package['activePresets'] + package['accountRestrictedActivePresets']:
             sizes.append({
                 'name': preset['description'],
                 'key': preset['keyName']
@@ -404,29 +423,26 @@ class HardwareManager(utils.IdentifierMixin, object):
             'extras': extras,
         }
 
-    @retry(exceptions.SoftLayerAPIError, logger=LOGGER)
+    @retry(logger=LOGGER)
     def _get_package(self):
         """Get the package related to simple hardware ordering."""
         mask = '''
-items[
-    keyName,
-    capacity,
-    description,
-    attributes[id,attributeTypeKeyName],
-    itemCategory[id,categoryCode],
-    softwareDescription[id,referenceCode,longDescription],
-    prices
-],
-activePresets,
-regions[location[location[priceGroups]]]
-'''
+            items[
+                keyName,
+                capacity,
+                description,
+                attributes[id,attributeTypeKeyName],
+                itemCategory[id,categoryCode],
+                softwareDescription[id,referenceCode,longDescription],
+                prices
+            ],
+            activePresets,
+            accountRestrictedActivePresets,
+            regions[location[location[priceGroups]]]
+            '''
 
         package_keyname = 'BARE_METAL_SERVER'
-        package = self.ordering_manager.get_package_by_key(package_keyname,
-                                                           mask=mask)
-        if package is None:
-            raise SoftLayer.SoftLayerError("Ordering package not found")
-
+        package = self.ordering_manager.get_package_by_key(package_keyname, mask=mask)
         return package
 
     def _generate_create_dict(self,
@@ -584,8 +600,33 @@ regions[location[location[priceGroups]]]
         """
 
         return self.hardware.createFirmwareUpdateTransaction(
-            bool(ipmi), bool(raid_controller), bool(bios), bool(hard_drive),
-            id=hardware_id)
+            bool(ipmi), bool(raid_controller), bool(bios), bool(hard_drive), id=hardware_id)
+
+    def wait_for_ready(self, instance_id, limit=14400, delay=10, pending=False):
+        """Determine if a Server is ready.
+
+        A server is ready when no transactions are running on it.
+
+        :param int instance_id: The instance ID with the pending transaction
+        :param int limit: The maximum amount of seconds to wait.
+        :param int delay: The number of seconds to sleep before checks. Defaults to 10.
+        """
+        now = time.time()
+        until = now + limit
+        mask = "mask[id, lastOperatingSystemReload[id], activeTransaction, provisionDate]"
+        instance = self.get_hardware(instance_id, mask=mask)
+        while now <= until:
+            if utils.is_ready(instance, pending):
+                return True
+            transaction = utils.lookup(instance, 'activeTransaction', 'transactionStatus', 'friendlyName')
+            snooze = min(delay, until - now)
+            LOGGER.info("%s - %d not ready. Auto retry in %ds", transaction, instance_id, snooze)
+            time.sleep(snooze)
+            instance = self.get_hardware(instance_id, mask=mask)
+            now = time.time()
+
+        LOGGER.info("Waiting for %d expired.", instance_id)
+        return False
 
 
 def _get_extra_price_id(items, key_name, hourly, location):
@@ -748,15 +789,13 @@ def _get_location(package, location):
         if region['location']['location']['name'] == location:
             return region
 
-    raise SoftLayer.SoftLayerError("Could not find valid location for: '%s'"
-                                   % location)
+    raise SoftLayer.SoftLayerError("Could not find valid location for: '%s'" % location)
 
 
 def _get_preset_id(package, size):
     """Get the preset id given the keyName of the preset."""
-    for preset in package['activePresets']:
-        if preset['keyName'] == size:
+    for preset in package['activePresets'] + package['accountRestrictedActivePresets']:
+        if preset['keyName'] == size or preset['id'] == size:
             return preset['id']
 
-    raise SoftLayer.SoftLayerError("Could not find valid size for: '%s'"
-                                   % size)
+    raise SoftLayer.SoftLayerError("Could not find valid size for: '%s'" % size)
